package com.coolimagingproject.basicImageProcessingOperations.operador.tratamientoImagenes;

import com.coolimagingproject.coolimaging.libreriaimagenes.imagen.publico.Imagen;

import java.awt.RenderingHints;
import java.awt.image.renderable.ParameterBlock;

import javax.media.jai.JAI;
import javax.media.jai.PlanarImage;

import com.coolimagingproject.basicImageProcessingOperations.operador.tratamientoImagenes.OperadorRotacion;

import com.coolimagingproject.coolimaging.libreriaimagenes.operador.publico.ConjuntoParametroOperador;
import com.coolimagingproject.coolimaging.libreriaimagenes.operador.publico.OperadorUnario;

/**
 * <p>
 * La clase OperadorRotacion representa un Operador que lleva a cabo la rotacin
 * de una imagen en un ngulo determinado, respecto a un centro determinado. Los
 * parmetros asociados que tiene el OperadorEscalado estn identificados
 * mediante un int, y son:
 * </p>
 * 
 * <ul>
 * <li>OperadorRotacion.PARAMETRO_IMAGEN_FUENTE: objeto tipo Imagen a ser
 * rotado.</li>
 * <li>OperadorRotacion.PARAMETRO_ANGULO_ROTACION: ngulo de rotacin, medido en
 * radianes, y en sentido horario (float).</li>
 * <li>OperadorRotacion.PARAMETRO_CENTRO_ROTACION_X: coordenada X del centro de
 * rotacin (float).</li>
 * <li>OperadorRotacion.PARAMETRO_CENTRO_ROTACION_Y: coordenada Y del centro de
 * rotacin (float).</li>
 * <li>OperadorRotacion.PARAMETRO_METODO_INTERPOLACION: mtodo de interpolacin
 * usado para ajustar la imagen tras el rotado (Interpolation).</li>
 * <li>OperadorRotacion.PARAMETRO_METODO_BORDE: mtodo usado para el tratamiento
 * del borde de la imagen cuando es rotada.</li>
 * <li>OperadorRotacion.PARAMETRO_ROTAR_Y_CENTRAR: indica si la imagen debe ser
 * centrada despus de ser rotada, para que no se salga de la pantalla
 * (booleano).</li>
 * </ul>
 * 
 * @see OperadorUnario
 * 
 * @author L.A Gonzlez Jaime
 * @author R.J Palma Durn
 * 
 */
public class OperadorRotacion extends OperadorUnario{
	// Parmetros de la operacin a nivel de JAI
	private ParameterBlock parametrosJAI = new ParameterBlock();
	// Almacena el mtodo usado para el tratamiento del borde en la operacin de
	// rotacin
	private RenderingHints hints;
	// Nombre, a nivel de JAI, del operador. Es usado cuando se llama al mtodo
	// JAI.create()
	protected String nombreOperador;
	// Indica si la imagen debe ser centrada tras ser rotada
	protected boolean centrarImagen;
	/**
	 * Identificador del parmetro que representa el ngulo de rotacin, medido
	 * en radianes y en sentido antihorario. El objeto asociado es de tipo
	 * float.
	 */
	public static final int PARAMETRO_ANGULO_ROTACION = 10;
	/**
	 * Identificador del parmetro que representa la coordenada X del centro de
	 * rotacin. El objeto asociado es de tipo float.
	 */
	public static final int PARAMETRO_CENTRO_ROTACION_X = 11;
	/**
	 * Identificador del parmetro que representa la coordenada Y del centro de
	 * rotacin. El objeto asociado es de tipo float.
	 */
	public static final int PARAMETRO_CENTRO_ROTACION_Y = 12;
	/**
	 * Identificador del parmetro que representa el mtodo de interpolacin
	 * usado para tratar la imagen tras la rotacin. El objeto asociado es de
	 * tipo Interpolation.
	 */
	public static final int PARAMETRO_METODO_INTERPOLACION = 13;
	/**
	 * Identificador del parmetro que representa el mtodo usado para el
	 * tratamiento del borde de la imagen en la rotacin. El objeto asociado es
	 * de tipo BorderExtender.
	 */
	public static final int PARAMETRO_METODO_BORDE = 14;
	/**
	 * Identificador del parmetro que indica si la imagen, tras ser rotada,
	 * debera ser trasladada para quedar centrada (su origen de coordenadas
	 * sera (0,0), y no aqul impuesto por la rotacin). El objeto asociado es
	 * de tipo boolean.
	 */
	public static final int PARAMETRO_ROTAR_Y_CENTRAR = 15;

	/**
	 * <p>
	 * Construye un objeto de tipo OperadorRotacion, con los parmetros
	 * especificados en el objeto ConjuntoParametroOperador parametros.
	 * </p>
	 * 
	 * <p>
	 * El objeto ConjuntoParametroOperador que recibe como argumento debe
	 * contener los siguientes objetos ParametroOperador:
	 * </p>
	 * 
	 * <ul>
	 * <li>{OperadorRotacion.PARAMETRO_IMAGEN_FUENTE,Imagen}: imagen a rotar.</li>
	 * <li>{OperadorRotacion.PARAMETRO_ANGULO_ROTACION,float}: ngulo de
	 * rotacin, medido en radianes, y en sentido horario.</li>
	 * <li>{OperadorRotacion.PARAMETRO_CENTRO_ROTACION_X,float}: coordenada X
	 * del centro de rotacin de la imagen.</li>
	 * <li>{OperadorRotacion.PARAMETRO_CENTRO_ROTACION_Y,float}: coordenada Y
	 * del centro de rotacin de la imagen.</li>
	 * <li>{OperadorRotacion.PARAMETRO_METODO_INTERPOLACION,Interpolation}:
	 * mtodo de interpolacin usado para el tratamiento de la imagen tras ser
	 * rotada.</li>
	 * <li>{OperadorRotacion.PARAMETRO_METODO_BORDE,BorderExtender}: mtodo
	 * usado para el tratamiento del borde de la imagen en la rotacin.</li>
	 * <li>{OperadorRotacion.PARAMETRO_ROTAR_Y_CENTRAR,boolean}: indica si la
	 * imagen, tras ser rotada, debera ser trasladada para quedar centrada (su
	 * origen de coordenadas sera (0,0), y no aqul impuesto por la rotacin).
	 * </ul>
	 * 
	 * @param parametros
	 *        objeto ConjuntoParametroOperador que almacena los parmetros
	 *        especificados anteriormente.
	 */
	public OperadorRotacion(ConjuntoParametroOperador parametros){
		super(parametros);
		this.nombreOperador = "rotate";
		this.parametrosJAI.addSource(parametros.getParametro(
				OperadorRotacion.PARAMETRO_IMAGEN_FUENTE).getValorParametro());
		this.parametrosJAI.add(parametros.getParametro(
				OperadorRotacion.PARAMETRO_CENTRO_ROTACION_X)
				.getValorParametro());
		this.parametrosJAI.add(parametros.getParametro(
				OperadorRotacion.PARAMETRO_CENTRO_ROTACION_Y)
				.getValorParametro());
		this.parametrosJAI
				.add(parametros.getParametro(
						OperadorRotacion.PARAMETRO_ANGULO_ROTACION)
						.getValorParametro());
		this.parametrosJAI.add(parametros.getParametro(
				OperadorRotacion.PARAMETRO_METODO_INTERPOLACION)
				.getValorParametro());
		this.centrarImagen = (Boolean)parametros.getParametro(
				OperadorRotacion.PARAMETRO_ROTAR_Y_CENTRAR).getValorParametro();
		this.hints = new RenderingHints(JAI.KEY_BORDER_EXTENDER, parametros
				.getParametro(OperadorRotacion.PARAMETRO_METODO_BORDE)
				.getValorParametro());
	}

	/*
	 * (non-Javadoc)
	 * @see utilidades.Operador#operar()
	 */
	public Object operar(){
		PlanarImage imagenIntermedia = JAI.create(this.nombreOperador,
				this.parametrosJAI, this.hints);
		if(this.centrarImagen){
			ParameterBlock parametrosTranslacion = new ParameterBlock();
			parametrosTranslacion.addSource(imagenIntermedia);
			parametrosTranslacion.add((float)(-imagenIntermedia.getMinX()));
			parametrosTranslacion.add((float)(-imagenIntermedia.getMinY()));
			parametrosTranslacion.add(null);
			return new Imagen(JAI.create("translate", parametrosTranslacion));
		}
		return new Imagen(imagenIntermedia);
	}
}
